/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpemenubar.h>
#include <qpe/config.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include "Mahjongg.h"
#include "Board.h"
#include <stdlib.h>
#include <time.h>
#include <qdir.h>
#include "ScoreListDlg.h"

#define TIME_TO_SEC(h,m,s) (h*3600+m*60+s)

//Score Files
#define SCORE_DIR       ".m-mahjongg"
#define C_SCORE_FNAME01 ".m-mahjongg/score01.dat"
#define C_SCORE_FNAME02 ".m-mahjongg/score02.dat"
#define C_SCORE_FNAME03 ".m-mahjongg/score03.dat"
#define C_SCORE_FNAME04 ".m-mahjongg/score04.dat"
#define C_SCORE_FNAME05 ".m-mahjongg/score05.dat"

Mahjongg::Mahjongg(QWidget *parent, const char *name, WFlags f) :
    QMainWindow(parent, name, f)
{
    /* initialize random number generator */
    srand(::time(0));

    /* set windows style and caption */
    setCaption("Mobile Mahjongg");

    /* setup score directory */
    QDir dir;
    if( !dir.exists( SCORE_DIR ) ){
      dir.mkdir( SCORE_DIR );
    }
    initHiScoreMng();

    /* create the play field as the central widget */
    m_Status = new CStatus();
    m_board = new Board(this);
    m_board->setStatus(m_Status);

    setCentralWidget(m_board);
    m_board->show();
    connect(m_board, SIGNAL(changed()), this, SLOT(enableItems()));
    connect(m_board, SIGNAL(gameOver()), this, SLOT(gameOver()));

    /* create the game/layout submenu */
    m_layoutMenu = new QPopupMenu(0, "layout_menu");
    m_layoutMenu->setCheckable(true);
    connect(m_layoutMenu, SIGNAL(activated(int)), this, SLOT(updateLayout(int)));
    m_layoutMenu->insertItem(tr("Mahjongg"), 0);
    m_layoutMenu->insertItem(tr("Pyramid"), 1);
    m_layoutMenu->insertItem(tr("Towers"), 2);
    m_layoutMenu->insertItem(tr("Triangle"), 3);
    m_layoutMenu->insertItem(tr("Walls"), 4);

    /* create the game submenu */
    m_gameMenu = new QPopupMenu(0, "game_menu");
    m_idMenu[0] = m_gameMenu->insertItem(tr("New"), this, SLOT(start()));
    m_idMenu[1] = m_gameMenu->insertItem(tr("AutoPlay"), m_board, SLOT(autoPlay()));
    m_idMenu[2] = m_gameMenu->insertItem(tr("Layout"), m_layoutMenu);
    m_gameMenu->insertSeparator();
    m_idMenu[3] = m_gameMenu->insertItem(tr("Hint"), this, SLOT(hint()));
    m_idMenu[4] = m_gameMenu->insertItem(tr("Undo"), this, SLOT(undo()), 0, 0x0100);
    m_idMenu[5] = m_gameMenu->insertItem(tr("Redo"), this, SLOT(redo()), 0, 0x0101);
    m_idMenu[6] = m_gameMenu->insertItem(tr("Give up"), this, SLOT(giveup()));
    m_gameMenu->insertSeparator();
    m_idMenu[7] = m_gameMenu->insertItem(tr("Hi-Score"), this, SLOT(showScore()));
    m_gameMenu->insertSeparator();
    m_idMenu[8] = m_gameMenu->insertItem(tr("Close"), this, SLOT(close()));

    /* create the menu */
    QPEMenuBar *menu = new QPEMenuBar(this);
    menu->insertItem(tr("Game"), m_gameMenu);

    /* read the configuration file */
    readConfig();

    /* check the menu entries of game size and game iq submenu */
    m_layoutMenu->setItemChecked(m_layout, true);
    m_board->updateLayout(m_layout);
    //enableItems();

    /* start a game automatically */
    //start();
    disableItems();
}

Mahjongg::~Mahjongg()
{
    writeConfig();
    deleteHiScoreMng();
    delete m_Status;
}

void Mahjongg::readConfig(void)
{
    Config cfg("Mahjongg");

    cfg.setGroup("Settings");

    int layout = cfg.readNumEntry("Layout", 0);
    if (layout <= 0) {
	m_layout = 0;
    } else if (layout >= 4) {
	m_layout = 4;
    } else {
	m_layout = layout;
    }
    m_board->updateLayout(m_layout);
}

void Mahjongg::writeConfig(void)
{
    Config cfg("Mahjongg");

    cfg.setGroup("Settings");
    cfg.writeEntry("Layout", m_layout);
}

void Mahjongg::updateLayout(int layout)
{
    m_layoutMenu->setItemChecked(m_layout, false);
    m_layout = layout;
    m_layoutMenu->setItemChecked(m_layout, true);
    writeConfig();
    m_board->updateLayout(layout);
    start();
}

void Mahjongg::start(void)
{
    m_board->start();
    enableItems();
}

void Mahjongg::hint(void)
{
    qApp->processEvents();
    m_board->displayHint();
    m_board->setCheated(true);
    enableItems();
}

void Mahjongg::undo(void)
{
    if (m_board->canUndo()) {
	m_board->undo();
	m_board->setCheated(true);
	enableItems();
    }
}

void Mahjongg::redo(void)
{
    if(m_board->canRedo()) {
	m_board->redo();
	enableItems();
    }
}

void Mahjongg::enableItems(void)
{
    int i;

    for( i = 0; i < MENU_ID_MAX; i++){
     m_gameMenu->setItemEnabled( m_idMenu[i],true);
    }
    m_gameMenu->setItemEnabled(0x100, m_board->canUndo());
    m_gameMenu->setItemEnabled(0x101, m_board->canRedo());
}

void Mahjongg::gameOver(void)
{

    if (m_board->tilesLeft()) {
	QMessageBox::information(this, tr("Mahjongg"),
				 tr("No more moves\npossible\nYou loose"));
    } else {
       struct tm *ptm;
       time_t now;
       bool ret;
       now = time(NULL);
       ptm = localtime(&now);

	int seconds = m_board->getTimeForGame();
	QString s,msg;
	m_pScoreMng[m_layout]->RegistHiScore(ptm,seconds);
	ret = m_pScoreMng[m_layout]->IsRankIn(seconds);
	if( ret ){
	  msg = tr("Congratulations!\nYou made it in\n%1\nHi-Score Registed !");
	} else {
	  msg = tr("Congratulations!\nYou made it in\n%1");
	}
	m_pScoreMng[m_layout]->SaveHiScore();

	s.sprintf("%02d:%02d:%02d", seconds / 3600, (seconds / 60) % 60, seconds % 60);
	QMessageBox::information(this, tr("Mobile Mahjongg"),msg.arg(s));
    }
    //start();
    m_Status->changeStatus(STS_TITLE);
}
//
// Init ScoreData
//
void Mahjongg::registHiScoreData( CScoreMng *pScoreMng )
{
  struct tm *ptm;
  time_t now;

  now = time(NULL);
  ptm = localtime(&now);
  pScoreMng->RegistHiScore( ptm, TIME_TO_SEC(0,20,0) );
  pScoreMng->RegistHiScore( ptm, TIME_TO_SEC(0,20,10) );
  pScoreMng->RegistHiScore( ptm, TIME_TO_SEC(0,20,20) );
  pScoreMng->RegistHiScore( ptm, TIME_TO_SEC(0,20,30) );
  pScoreMng->RegistHiScore( ptm, TIME_TO_SEC(0,20,40) );
  pScoreMng->SaveHiScore();
  
}
//
// Create Score Managers
//
void Mahjongg::initHiScoreMng()
{
    
  // create score Manager
  m_pScoreMng[0] = new CScoreMng( C_SCORE_FNAME01 );
  m_pScoreMng[1] = new CScoreMng( C_SCORE_FNAME02 );
  m_pScoreMng[2] = new CScoreMng( C_SCORE_FNAME03 );
  m_pScoreMng[3] = new CScoreMng( C_SCORE_FNAME04 );
  m_pScoreMng[4] = new CScoreMng( C_SCORE_FNAME05 );

  for(int i=0; i < CATEGORY_NUM; i++){
    m_pScoreMng[i]->SetOrder( false );
    if( !m_pScoreMng[i]->DownLoadHiScore() ){
      // read scorefile failed.
      registHiScoreData( m_pScoreMng[i] );
    }    
  }
  
}
//
// Delete Score Managers
//
void Mahjongg::deleteHiScoreMng()
{
  for(int i=0; i < CATEGORY_NUM; i++){
    delete m_pScoreMng[i];
  }  
}

void Mahjongg::showScore(void)
{
    CScoreListDlg win(this, "Options", true);
    win.SetScoreMng(&m_pScoreMng[0]);
    win.exec();
}
void Mahjongg::disableItems(void)
{
  int i;
  for( i = 0; i < MENU_ID_MAX; i++){
    m_gameMenu->setItemEnabled( m_idMenu[i],false);
  }

  m_gameMenu->setItemEnabled( m_idMenu[0],true);
  m_gameMenu->setItemEnabled( m_idMenu[2],true);
  m_gameMenu->setItemEnabled( m_idMenu[7],true);
  m_gameMenu->setItemEnabled( m_idMenu[8],true);
}
void Mahjongg::giveup(void)
{
 
  disableItems();
  m_Status->init();
  m_board->repaint(0, 0, width(), height(), false);
}
